﻿using System;
using AopAlliance.Intercept;
using AspectSharp.Builder;

namespace AspectSharp.Test
{
    class Program
    {
        static void Main(string[] args)
        {
            StandardCall();
            AopCall();
            Console.ReadKey();
        }
        //正常的调用，不使用AOP拦截
        private static void StandardCall()
        {
            RequestProcessor rp = new RequestProcessor();
            rp.Process();
            Console.WriteLine();
        }
        //使用AOP拦截后的调用
        private static void AopCall()
        {
            //AOP的拦截配置，这些配置可以放入配置文件中
            //import: 导入命名空间，比如拦截器所在的命名空间
            //aspect: 定义一个拦截的aspect, for指令用于指示需要拦截的类所在的命名空间
            //pointcut: 定义一个拦截点，使用method指定需要拦截的方法，还可以使用property指定需要拦截的属性，
            //    或者使用propertyread、propertywrite等用于拦截属性的getter、setter操作，
            //    被拦截的方法需要是virtual类型的，否则无法实现拦截（跟Castle Dynamic Proxy机制相关）
            //advice: 定义拦截器
            string config = @"
import AspectSharp.Test

aspect processor for [AspectSharp.Test] 
    pointcut method(* Process())
        advice(LoggerInterceptor)
    end
end";
            AspectLanguageEngineBuilder engineBuilder = new AspectLanguageEngineBuilder(config);
            AspectEngine engine = engineBuilder.Build();
            //使用AspectSharp包装，AspectSharp将根据配置生成代理对象，在代理对象上使用拦截器实现AOP拦截处理
            RequestProcessor rp = engine.WrapClass(typeof(RequestProcessor)) as RequestProcessor;
            rp.Process();
            Console.WriteLine();
        }
    }
    //测试用的目标类
    public class RequestProcessor
    {
        //需要是virtual的方法才能拦截
        public virtual void Process()
        {
            Console.WriteLine("Hello AspectSharp");
        }
    }
    //记录日志用的拦截器
    public class LoggerInterceptor : IMethodInterceptor
    {
        public object Invoke(IMethodInvocation invocation)
        {
            Console.WriteLine("Before {0} on {1}", invocation.Method.Name, invocation.Method.DeclaringType);
            object returnVal = invocation.Proceed();
            Console.WriteLine("After {0} on {1}", invocation.Method.Name, invocation.Method.DeclaringType);
            return returnVal;
        }
    }
}